package com.sun.rowset;

import java.io.ObjectInputStream;
import java.io.OutputStream;
import java.io.InputStream;
import java.io.Reader;
import java.io.Writer;
import java.sql.ResultSet;
import javax.sql.rowset.spi.SyncFactory;
import java.io.IOException;
import java.util.Hashtable;
import java.sql.SQLException;
import javax.sql.rowset.spi.SyncProvider;
import com.sun.rowset.internal.WebRowSetXmlWriter;
import com.sun.rowset.internal.WebRowSetXmlReader;
import javax.sql.rowset.WebRowSet;

public class WebRowSetImpl extends CachedRowSetImpl implements WebRowSet
{
    private WebRowSetXmlReader xmlReader;
    private WebRowSetXmlWriter xmlWriter;
    private int curPosBfrWrite;
    private SyncProvider provider;
    static final long serialVersionUID = -8771775154092422943L;
    
    public WebRowSetImpl() throws SQLException {
        this.xmlReader = new WebRowSetXmlReader();
        this.xmlWriter = new WebRowSetXmlWriter();
    }
    
    public WebRowSetImpl(Hashtable hashtable) throws SQLException {
        try {
            this.resBundle = JdbcRowSetResourceBundle.getJdbcRowSetResourceBundle();
        }
        catch (IOException cause) {
            throw new RuntimeException(cause);
        }
        if (hashtable == null) {
            throw new SQLException(this.resBundle.handleGetObject("webrowsetimpl.nullhash").toString());
        }
        this.provider = SyncFactory.getInstance((String)hashtable.get("rowset.provider.classname"));
        return;
    }
    
    @Override
    public void writeXml(ResultSet set, Writer writer) throws SQLException {
        this.populate(set);
        this.curPosBfrWrite = this.getRow();
        this.writeXml(writer);
    }
    
    @Override
    public void writeXml(Writer wrt) throws SQLException {
        if (this.xmlWriter != null) {
            this.curPosBfrWrite = this.getRow();
            this.xmlWriter.writeXML(this, wrt);
            return;
        }
        throw new SQLException(this.resBundle.handleGetObject("webrowsetimpl.invalidwr").toString());
    }
    
    @Override
    public void readXml(Reader reader) throws SQLException {
        try {
            if (reader != null) {
                this.xmlReader.readXML(this, reader);
                if (this.curPosBfrWrite == 0) {
                    this.beforeFirst();
                }
                else {
                    this.absolute(this.curPosBfrWrite);
                }
            }
            else {
                throw new SQLException(this.resBundle.handleGetObject("webrowsetimpl.invalidrd").toString());
            }
        }
        catch (Exception ex) {
            throw new SQLException(ex.getMessage());
        }
    }
    
    @Override
    public void readXml(InputStream iStream) throws SQLException, IOException {
        if (iStream != null) {
            this.xmlReader.readXML(this, iStream);
            if (this.curPosBfrWrite == 0) {
                this.beforeFirst();
            }
            else {
                this.absolute(this.curPosBfrWrite);
            }
            return;
        }
        else {
            throw new SQLException(this.resBundle.handleGetObject("webrowsetimpl.invalidrd").toString());
        }
    }
    
    @Override
    public void writeXml(OutputStream oStream) throws SQLException, IOException {
        if (this.xmlWriter != null) {
            this.curPosBfrWrite = this.getRow();
            this.xmlWriter.writeXML(this, oStream);
            return;
        }
        throw new SQLException(this.resBundle.handleGetObject("webrowsetimpl.invalidwr").toString());
    }
    
    @Override
    public void writeXml(ResultSet set, OutputStream outputStream) throws SQLException, IOException {
        this.populate(set);
        this.curPosBfrWrite = this.getRow();
        this.writeXml(outputStream);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        try {
            this.resBundle = JdbcRowSetResourceBundle.getJdbcRowSetResourceBundle();
        }
        catch (IOException cause) {
            throw new RuntimeException(cause);
        }
    }
}
